@@ -15,6 +15,7 @@ import multiprocessing
 import os
 import pathlib
 import re
+from string import Template
 from typing import ClassVar, List
 
 import runez
@@ -491,11 +492,47 @@ class ModuleBuilder:
     def cfg_version(self, default):
         return PPG.config.get_value("%s-version" % self.m_name) or default
 
+    def cfg_http_headers(self):
+        if config_http_headers := PPG.config.get_value("%s-http-headers" % self.m_name):
+            expanded_http_headers = {}
+            for header_dict in config_http_headers:
+                for key, value in header_dict.items():
+                    expanded_http_headers[os.path.expandvars(key)] = os.path.expandvars(value)
+
+            return expanded_http_headers
+
+    def cfg_url(self, version):
+        if config_url := PPG.config.get_value("%s-url" % self.m_name):
+            url_template = Template(config_url)
+            url_subbed = url_template.substitute(version=version)
+            return os.path.expandvars(url_subbed)
+
+    def cfg_src_suffix(self):
+        return PPG.config.get_value("%s-src-suffix" % self.m_name)
+
+    def cfg_configure(self, deps_lib_dir, deps_lib64_dir):
+        if configure := PPG.config.get_value("%s-configure" % self.m_name):
+            configure_template = Template(configure)
+            return configure_template.substitute(lib_dir=deps_lib_dir, lib64_dir=deps_lib64_dir)
+
+    def cfg_patches(self):
+        return PPG.config.get_value("%s-patches" % self.m_name)
+
     @property
     def url(self):
         """Url of source tarball, if any"""
         return ""
 
+    @property
+    def headers(self):
+        """Headers for connecting to source url, if any"""
+        return self.cfg_http_headers()
+
+    @property
+    def src_suffix(self):
+        """Suffix of src archive for when URL doesn't end in the file extension"""
+        return self.cfg_src_suffix()
+
     @property
     def version(self):
         """Version to use"""
@@ -507,9 +544,20 @@ class ModuleBuilder:
         return self.setup.folders.deps
 
     @property
-    def deps_lib(self):
+    def deps_lib_dir(self):
         return self.deps / "lib"
 
+    @property
+    def deps_lib64_dir(self):
+        return self.deps / "lib64"
+
+    @property
+    def deps_lib_dirs(self):
+        lib_dirs = [self.deps_lib_dir]
+        if self.deps_lib64_dir.exists():
+            lib_dirs.append(self.deps_lib64_dir)
+        return lib_dirs
+
     def xenv_CPATH(self):
         folder = self.deps / "include"
         if folder.exists():
@@ -522,7 +570,7 @@ class ModuleBuilder:
 
     def xenv_LDFLAGS(self):
         if self.modules.selected:
-            yield f"-L{self.deps_lib}"
+            yield from (f"-L{lib_dir}" for lib_dir in self.deps_lib_dirs)
 
     def xenv_PATH(self):
         yield f"{self.deps}/bin"
@@ -536,7 +584,7 @@ class ModuleBuilder:
     def xenv_PKG_CONFIG_PATH(self):
         yield from os.environ.get("PKG_CONFIG_PATH", "").split(":")
         if self.modules.selected:
-            yield f"{self.deps_lib}/pkgconfig"
+            yield from (f"{lib_dir}/pkgconfig" for lib_dir in self.deps_lib_dirs)
 
     def _do_run(self, program, *args, fatal=True, env=None):
         return runez.run(program, *args, passthrough=self._log_handler, stdout=None, stderr=None, fatal=fatal, env=env)
@@ -607,8 +655,16 @@ class ModuleBuilder:
                         self._finalize()
                         return
 
+                # Some URL's may not end in file extension, such as with redirects.
+                # Github releases asset endpoint is this way .../releases/assets/48151
+
                 # Split on '#' for urls that include a checksum, such as #sha256=... fragment
                 basename = runez.basename(self.url, extension_marker="#")
+                if not basename.endswith((".zip", ".tar.gz")):
+                    suffix = self.src_suffix or ".tar.gz"
+                    suffix = ".%s" % (suffix.strip("."))  # Ensure it starts with a dot (in case config forgot leading dot)
+                    basename = f"{self.m_name}-{self.version}{suffix}"
+
                 path = self.setup.folders.sources / basename
                 if not path.exists():
                     proxies = {}
@@ -618,7 +674,8 @@ class ModuleBuilder:
                     https_proxy = os.environ.get("HTTPS_PROXY") or os.environ.get("https_proxy")
                     if https_proxy:
                         proxies["https"] = https_proxy
-                    RestClient().download(self.url, path, proxies=proxies)
+
+                    RestClient().download(self.url, path, proxies=proxies, headers=self.headers)
 
                 runez.decompress(path, self.m_src_build, simplify=True)
 
@@ -639,6 +696,7 @@ class ModuleBuilder:
                         folder = folder / self.m_build_cwd
 
                     with runez.CurrentFolder(folder):
+                        self._apply_patches()
                         self._prepare()
                         func()
                         self._finalize()
@@ -652,6 +710,15 @@ class ModuleBuilder:
                     else:
                         os.environ[k] = v
 
+    def _apply_patches(self):
+        if patches := self.cfg_patches():
+            for patch in patches:
+                if runez.DRYRUN:
+                    print(f"Would apply patch: {patch}")
+                else:
+                    print(f"Applying patch: {patch}")
+                    patch_file(patch["file"], patch["regex"], patch["replacement"])
+
     def _get_env_vars(self):
         """Yield all found env vars, first found wins"""
         result = {}
@@ -747,10 +814,12 @@ class PythonBuilder(ModuleBuilder):
         # Some libs get funky permissions for some reason
         super()._prepare()
         self.setup.ensure_clean_folder(self.install_folder)
-        for path in runez.ls_dir(self.deps_lib):
-            if not path.name.endswith(".la"):
-                expected = 0o755 if path.is_dir() else 0o644
-                current = path.stat().st_mode & 0o777
-                if current != expected:
-                    LOG.info("Corrected permissions for %s (was %s)", runez.short(path), oct(current))
-                    path.chmod(expected)
+
+        for lib_dir in self.deps_lib_dirs:
+            for path in runez.ls_dir(lib_dir):
+                if not path.name.endswith(".la"):
+                    expected = 0o755 if path.is_dir() else 0o644
+                    current = path.stat().st_mode & 0o777
+                    if current != expected:
+                        LOG.info("Corrected permissions for %s (was %s)", runez.short(path), oct(current))
+                        path.chmod(expected)
